import htmlc from './lib/htmlColors.js';
import printc from './lib/printc.js';
import readline from 'readline';
import fs from 'fs';
import path from 'path';
import { fileURLToPath } from 'url';
import readlineSync from 'readline-sync';

let __temp;
const __filename = fileURLToPath(import.meta.url);
const __dirname = path.dirname(__filename);

let fqa = {
  home: process.env.HOME || process.env.HOMEPATH || process.env.USERPROFILE,
  path: {
    raw: process.env.PATH,
    parsed: process.env.PATH ? process.env.PATH.split(':') : []
  },
  colors: {}
};

__temp = {
  term: {
    warn: 3,
    error: 1,
    text: 4,
    prompt: 15
  },
  html: {
    warn: htmlc['Yellow'],
    error: htmlc['Red'],
    text: htmlc['White'],
    prompt: htmlc['Gray'],
  },
  none: {
    warn: -1,
    error: -1,
    text: -1,
    prompt: -1,
  },
};

const configPath = path.join(fqa.home, '.config', 'jshell', 'config.json');
const historyFile = process.env.JSHELL_HISTORY || `${fqa.home}/.history`;

if (!fs.existsSync(configPath)) {
  process.stdout.write(printc('FF0000', `No config file was found in ${fqa.home}/.config/jshell/config.json.` + '\n'));
  process.stdout.write('Do you want to generate a standart config now? (Y/n)' + '\n');

  const userResponse = readlineSync.question('> ');

  if (userResponse.toLowerCase() !== 'y' && userResponse !== '') {
    process.stdout.write('JSH cannot continue without a config file.' + '\n');
    process.stdout.write(printc('FF0000', 'JSH Fatal. exitting'));
    process.exit(1);
  }
  fs.mkdirSync(path.dirname(configPath), { recursive: true });
  const defaultConfig = {
    colorMode: 'term',
    prompt: 'jsh> ',
    exitPhrase: 'exit',
    customCommandPrefix: '.'
  };
  fs.writeFileSync(configPath, JSON.stringify(defaultConfig, null, 2));
}

const config = JSON.parse(fs.readFileSync(configPath, 'utf-8'));
if (!(config.colorMode in __temp)) {
  process.stdout.write(printc('FFFFFFF', 'Invalid colorMode in config. (term, html, none)'));  
  process.stdout.write(printc('FF0000', 'JSH Fatal. exitting'))
  process.exit(1);
}

fqa.colors = __temp[config.colorMode];
__temp = undefined;

const shctx = {
  fqa: fqa,
  config: config
}

readline.emitKeypressEvents(process.stdin);
if (process.stdin.isTTY) process.stdin.setRawMode(true);
let shv = {
  line: '',
  chars: [],
};

process.stdin.on('keypress', async (char, key) => {
  if (key.sequence === '\u0003') {
    process.stdout.write(config.exitPhrase || 'exit');
    process.exit();
  }
  if (key.name === 'return') {
    process.stdout.write('\n');
    if (shv.line == '' || shv.line.startsWith('//')) {
      // skip
    } else if (shv.line.startsWith('./')) {
      // LOCAL command
      process.stdout.write(printc(fqa.colors.warn, 'Executing as an LOCAL command' + '\n'));
    } else if (shv.line.startsWith(config.customCommandPrefix || '.')) {
      process.stdout.write(printc(fqa.colors.warn, 'Executing as an JSH command') + '\n');
      // JSH command
      const commandPath = path.join(__dirname, 'builtin_commands', `${shv.line.slice(config.customCommandPrefix.length)}`);

      let commandFile = commandPath;
      if (!fs.existsSync(commandFile)) {
        commandFile = `${commandPath}.js`;
        if (!fs.existsSync(commandFile)) {
          commandFile = `${commandPath}.ts`;
        }
      }

      if (fs.existsSync(commandFile)) {
        await import(commandFile).then(async command => {
          if (typeof command.default === 'function') {
            await command.default(shctx);
          } else {
            process.stdout.write(printc(fqa.colors.error, 'Loaded module is not a function.' + '\n'));
          }
        }).catch(err => {
          process.stdout.write(printc(fqa.colors.error, 'Failed to execute JSH command:' + '\n'));
          console.error(err);
        });
      } else {
        process.stdout.write(printc(fqa.colors.error, 'No such JSH command.' + '\n'));
      }
    } else {
      // UNIX command
      process.stdout.write(printc(fqa.colors.warn, 'Executing as an UNIX command' + '\n'));
      const command = shv.line.split(' ')[0]
      let found = false;
      for (const dir of fqa.path.parsed) {
        const commandFile = path.join(dir, command);
        if (fs.existsSync(commandFile) && fs.statSync(commandFile).isFile()) {
          process.stdout.write(printc(fqa.colors.text, `Found executable: ${commandFile}` + '\n'));
          found = true;
          break;
        }
      }
      if (!found) {
        process.stdout.write(printc(fqa.colors.error, 'No such command.') + '\n');
      }
      const args = shv.line.split(' ').slice(1);
      process.stdout.write(printc(fqa.colors.text, `Args: ${JSON.stringify(args)}`) + '\n');
    }
    process.stdout.write(printc(fqa.colors.prompt, config.prompt));
    shv.line = '';
    shv.chars = [];
  } else if (key.name === 'backspace') {
    if (shv.line.length > 0) {
      shv.line = shv.line.slice(0, -1);
      shv.chars.pop();
      process.stdout.clearLine();
      process.stdout.cursorTo(0);
      process.stdout.write(printc(fqa.colors.prompt, config.prompt) + printc(fqa.colors.text, shv.chars.map(c => c.render).join('')));
    }
  } else {
    shv.line += char;
    const r = printc(fqa.colors.text, char);
    shv.chars.push({ render: r, rawKey: key, rawChar: char});
    process.stdout.write(r);
  }
});
process.stdout.write(printc(fqa.colors.prompt, config.prompt));